package com.gic.haoban.manage.service.task.friend;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.clerk.api.service.PowerService;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.ClerkMainStoreRelatedApiService;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.haoban.manage.service.task.BaseSyncOperation;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberOpenCardBusinessService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class FriendSyncNewOperation implements BaseSyncOperation {
    private static final Logger logger = LoggerFactory.getLogger(FriendSyncNewOperation.class);
    @Autowired
    private PreDealService preDealService;
    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private ClerkMainStoreRelatedApiService clerkMainStoreRelatedApiService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private StaffClerkRelationService staffClerkRelationService;
    @Autowired
    private ClerkService clerkService;

    @Autowired
    private PowerService powerService;
    @Autowired
    private MemberOpenCardBusinessService memberOpenCardBusinessService;

    @Autowired
    private FriendClerkSyncNewOperation friendClerkSyncNewOperation;

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("第三方好友合并：{}", JSONObject.toJSONString(dataPre));
        String reason = "成功";
        boolean dealFlag = true;
        try {
            String wxEnterpriseId = dataPre.getWxEnterpriseId();
            String wxUserId = dataPre.getpDataId();
            String taskId = dealParamMqDTO.getTaskId();
            String externalUserId = dataPre.getDataContent();

            ExternalUserDTO externalUserDTO = memberUnionidRelatedApiService.getExternalUseridInfo(wxEnterpriseId, wxUserId, externalUserId);
            if (externalUserDTO == null) {
                dealFlag = false;
                reason = "好友不存在";
                return;
            }
            //relationkey
            String relationKey = SecureUtil.md5(wxUserId + "_" + externalUserDTO.getName() + "_" + externalUserDTO.getAddCreateTime());
            logger.info("relationKey：{}，{}", relationKey, externalUserDTO.getExternalUserId());
            // 根据关联键relationkey查看自建应用 是否有数据
            TabHaobanPreDealLog selfPre = preDealService.getByRelationKey(taskId, PreDealTypeEnum.self_friend.getVal(), relationKey);
            if (null == selfPre) {
                logger.info("用户的自建应用不存在该数据：{},{}", JSONObject.toJSONString(dataPre), JSONObject.toJSONString(externalUserDTO));
                dealFlag = false;
                reason = "用户的自建应用不存在该数据：" + relationKey;
                return;
            }
            ExternalUserDTO selfUserData = JSON.parseObject(selfPre.getDataContent(), ExternalUserDTO.class);
            externalUserDTO.setUnionId(selfUserData.getUnionId());
            //序列化数据
            String ret = dealMerge(externalUserDTO, dataPre);
            if (StringUtils.isNotBlank(ret)) {
                logger.info("好友创建失败：{},{}", ret, JSONObject.toJSONString(externalUserDTO));
                dealFlag = false;
                reason = ret + ":" + relationKey;
            }

            //更新状态
            preDealService.updateFriendStatusByDataId(taskId, dataPre.getDataId(), PreDealStatusEnum.computed.getVal(), reason, externalUserDTO.getMemberId(), relationKey);
            dealSuccess(taskId, dataPre.getDataId(), dataPre.getpDataId(), wxEnterpriseId);

        } catch (Exception e) {
            e.printStackTrace();
            logger.info("同步失败：{},{}", JSONObject.toJSONString(dataPre), e);
            reason = "第三方好友处理异常";
            dealFlag = false;
        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dataPre.getDataId(), dataPre.getpDataId(), reason);
            }
        }
    }

    private String dealMerge(ExternalUserDTO externalUserDTO, TabHaobanPreDealLog dataPre) {
        String wxEnterpriseId = dataPre.getWxEnterpriseId();
        String wxUserId = dataPre.getpDataId();
        String taskId = dataPre.getTaskId();

        //staff信息
        StaffDTO staff = staffApiService.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);
        if (staff == null) {
            return "成员不存在";
        }
        ClerkMainStoreRelatedDTO mainStore = clerkMainStoreRelatedApiService.getWxEnterpriseIdAndStaffId(wxEnterpriseId, staff.getStaffId());
        if (mainStore == null) {
            return "主门店不存在";
        }
        StaffClerkRelationDTO staffClerkRelationDTO = staffClerkRelationService.getOneBindByStoreId(staff.getStaffId(), mainStore.getStoreId());
        if (staffClerkRelationDTO == null) {
            return "未绑定导购";
        }

        externalUserDTO.setWxEnterpriseId(wxEnterpriseId);
        externalUserDTO.setEnterpriseId(staffClerkRelationDTO.getEnterpriseId());
        externalUserDTO.setStoreId(mainStore.getStoreId());
        externalUserDTO.setClerkId(staffClerkRelationDTO.getClerkId());
        externalUserDTO.setWxUserId(wxUserId);
        externalUserDTO.setStaffId(staffClerkRelationDTO.getStaffId());

        String clerkCode = staffClerkRelationDTO.getClerkCode();
        ClerkDTO clerk = clerkService.getClerkByClerkCode(staffClerkRelationDTO.getEnterpriseId(), clerkCode);
        if (clerk == null) {
            return "导购不存在:" + clerkCode;
        }
        String qrcodeParam = clerk.getQrcodeParam();
        if (StringUtils.isBlank(qrcodeParam)) {
            powerService.saveClerkQrCode(clerk.getClerkId());
            clerk = clerkService.getClerkByClerkCode(staffClerkRelationDTO.getEnterpriseId(), clerkCode);
            qrcodeParam = clerk.getQrcodeParam();
        }
        if (StringUtils.isBlank(qrcodeParam)) {
            return "导购码不存在：" + clerkCode;
        }
        MemberDTO member = null;
        if (clerk != null && StringUtils.isNotBlank(qrcodeParam) && StringUtils.isNotBlank(externalUserDTO.getUnionId())) {
            member = memberOpenCardBusinessService.getMemberByQywxChannelParam(staffClerkRelationDTO.getEnterpriseId(),
                    externalUserDTO.getUnionId(), "", externalUserDTO.getName(), externalUserDTO.getAvatar(), qrcodeParam);
        }
        if (member == null) {
            return "会员不存在:" + externalUserDTO.getUnionId();
        }
        externalUserDTO.setMemberId(member.getMemberId());
        return memberUnionidRelatedApiService.addExternal(externalUserDTO);
    }

    @Override
    public void dealException(String taskId, String dataId, String pDataId, String reason) {
        preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
        checkDepartmentTask(taskId, pDataId);
    }

    @Override
    public void dealSuccess(String taskId, String dataId, String pDataId, String wxEnterpriseId) {
        //校验成功是否都
        checkDepartmentTask(taskId, pDataId);
    }

    /**
     * 自定义处理流程
     *
     * @param taskId
     * @param pDataId
     */
    public void checkDepartmentTask(String taskId, String pDataId) {
        //校验是否都已经处理完成
        boolean b = preDealService.checkFriendTaskByPDataId(taskId, pDataId, PreDealTypeEnum.friend.getVal());
        if (!b) {
            return;
        }
        logger.info("同步第三方服务商好友：{},{}", taskId, pDataId);
        String key = "haoban_sync_third_friend_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanPreDealLog preDealLog = preDealService.getByDataId(taskId, pDataId, PreDealTypeEnum.friend_clerk.getVal(), PreDealStatusEnum.dealing.getVal());
        if (preDealLog != null) {
            preDealService.updateStatusByDataId(taskId, pDataId, PreDealTypeEnum.friend_clerk.getVal(), PreDealStatusEnum.computed.getVal(), "成功");
            this.checkDepartmentTask(taskId);
        }
        RedisUtil.unlock(key);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        friendClerkSyncNewOperation.checkDepartmentTask(taskId);
    }


}
